###########################################################################
# File:				calendar.tcl
# Author:			M Shaw
# Purpose:			Tcl Gregorian Calendar. 
#					Dual console and includable proc lib
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 23/Dec/2017	First working version (includable)
# 1.01 13/Nov/2019	Updated
###########################################################################
# Notes:
# Syntax: ticol.exe calendar.tcl jan 2010
#
# Supports year 1000 AD to 9999 AD
# Ported from C++
# 11ms CGI render time on an i5
#
# Display could be sped up by using a string and [append] or [store]
# instead of iterating output in an array but it is fast enough for 
# practical purposes at present
#
# We use: set month [array find months $argv(1) -nocase -length 3]
# to index months but we could, of course, invert the array and index
# directly
###########################################################################
# extern header_sent
#puts "Header sent? [item header_sent 0]"
/*
if {! [item header_sent 1]} {			# If var not set (e.g. cmd mode then we fake that it has been sent already)
	puts "Content-type: text/html\r\n\r\n"
	puts "<html><head><title>Tcl Calendar</title></head>
	<body bgcolor=\"#000000\" text=\"white\"><pre><font size=7>"
}
#*/

option expression off
option echo on
#option breakpoint on
#option stacktrace on

#puts "DEBUG: ENTER calendar.tcl main argc:$argc"

#    Ja  Fe  Ma  Ap  Ma  Ju  Jy  Au  Se  Oc  No  De
#	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
#	{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},

# Re-entry #
#undef calendar -nocomplain

if {! [defined calendar]} {
	proc calendar {month_in year_in {format {}}} {
		#########################################
		# [in]		Month (Base 1)
		# [in]		Year (Base 1)
		# [in,opt]	-html
		#########################################
		array set mon {
		   -1 ERROR
			0 JANUARY
			1 FEBRUARY
			2 MARCH
			3 APR
			4 MAY
			5 JUN
			6 JULY
			7 AUGUST
			8 SEPTEMBER
			9 OCTOBER
			10 NOVEMBER
			11 DECEMBER
		} #-const

		array set days_in_month {
			0,0		31 # Ja
			0,1		28 # Fe
			0,2		31 # Ma
			0,3		30 # Ap
			0,4		31 # Ma
			0,5		30 # Ju
			0,6		31 # Jy
			0,7		31 # Au
			0,8		30 # Se
			0,9		31 # Oc
			0,10	30 # No
			0,11 	31 # De
			1,0		31 # Ja
			1,1		29 # Fe
			1,2		31 # Ma
			1,3		30 # Ap
			1,4		31 # Ma
			1,5		30 # Ju
			1,6		31 # Jy
			1,7		31 # Au
			1,8		30 # Se
			1,9		31 # Oc
			1,10	30 # No
			1,11	31 # De
		}

		if {<= $month_in 0} {
			return
		}
		if {< $year_in 0} {
			return
		}
		
		set opt_html $::false
		set br ""
		set sp " "

		if {eq $format "-html"} {
			set opt_html $::true
			#set br "<br>"				# Not nesc now with <pre>
			#set sp "&nbsp;"
		}

		set leap [is_leapyear $year_in]
		array set days {
			0 0
			1 0
			2 0
			3 0
			4 0
			5 0
			6 0
		}

		#-- Get today's values #
		set today [date -day]		# For HTML highlighting
		set this_month [date -month]
		set this_year [date -year]

		#printf "DEBUG: Check day of week 23/Dec/2017 should be 6: %li\r\n" [day_of_week 23 12 2017]
		if {< $year_in 1752} {
			textcolor cyan
			puts "$br\r\n Gregorian Calendar for:" -nonewline
			textcolor
		}
		printf "$br\r\n %-21s  %4i $br\r\n" $mon([- $month_in 1]) $year_in 
		printf " Sun Mon Tue Wed Thu Fri Sat $br\r\n"

		for {set i 0} {< $i $days_in_month($leap,[- $month_in 1])} {++ i } {

			#puts "DEBUG: Calling \[day_of_week $i [- $month_in 1] $year_in\]"
			set dayofweek [day_of_week [+ $i 1] $month_in $year_in]
			set days([% $dayofweek 7]) [+ $i 1]
			
			#printf "DEBUG: dow:%li i==%li set@pos:%li == %li\r\n" $dayofweek $i [+ [% $dayofweek 7] 1] [+ $i 1]
			if {>= $dayofweek 6} {
				for {set j 0} {< $j 7} {++ j} {
					if {!= $days($j) 0} {
						# printf "debug: j:$j days(j):'$days($j)', today:'$today' '$days($j)'<br>"	
						# We only highlight today if displaying this month and year
						if {[&& [&& [== $this_year $year_in] [== $this_month $month_in]] [== $today $days($j)]] } {
							if {== $opt_html $::true} {
								printf "<font color=\"red\">"
								printf "$sp$sp%2i" $days($j)
								printf "</font>"
							} else {
								textcolor red
								printf "$sp$sp%2i" $days($j)
								textcolor
							}
						} else {
							printf "$sp$sp%2i" $days($j)
						}
					} else {
						printf "$sp$sp$sp$sp"
					}
				}
				printf " $br\r\n"									# Newline
				for {set j 0} {< $j 7} {++ j} {set days($j) 0}		# Zeroise day data for the week
			}
		}
		# Print remainder #
		for {set j 0} {< $j 7} {++ j} {
			if {$days($j)} {
				if {[&& [&& [== $this_year $year_in] [== $this_month $month_in]] [== $today $days($j)]] } {
					if {== $opt_html $::true} {
						printf "<font color=\"red\">"
						printf "$sp$sp%2i" $days($j)
						printf "</font>"
					} else {
						textcolor red
						printf "$sp$sp%2i" $days($j)
						textcolor
					}
				} else {
					printf "  %2i" $days($j)
				}
			} else {
				printf "    "
			}
		}
		printf " $br\r\n"
	}
}

# This section will run as a standalone script using argc, argv #

array set months {
	1 January
	2 February
	3 March
	4 April
	5 May
	6 June
	7 July
	8 August
	9 September
	10 October
	11 November
	12 December
}

# argv() ###################
# argv(0) == Script name
# argv(1+) == args
############################

#puts "DEBUG: calendar.tcl main argc:$argc"

unset month year -nocomplain						# Re-entrancy
set month 0
set year 0

if { [array is_set argv] } {						# It is fatal to address a non-existent array var
	#puts "DEBUG: argv1 $argv(0) argc=$argc"
	if {< $argc 3} {								# Just quit so we can include this lib for HTML CGI
		#puts "Syntax is calendar month year"
		return -code ok								# We must return code OK for CGI lib inclusion (will not return)
	} else {
		if {eq $argv(1) -} {
			return 
		}
		set month [array find months $argv(1) -nocase -length 3]	# Useful test for [array find] (name->integer)
		if {eq $month ""} {
			set month $argv(1)
		}
		set year $argv(2)
	}
}

halt

if {&& [> $month 0] [> $year 0]} {
	calendar $month $year								# Base 1 args
} else {
	# Comment-out for silent include mode
	#puts "calendar: Syntax is calendar month year"
}

/*
if {! [item header_sent 1]} {			# If var not set (e.g. cmd mode then we fake that it has been sent already)
	puts "</font></pre></body></html>"
}
# */

array unset months -nocomplain
unset month year -nocomplain

#puts "DEBUG: EXIT calendar.tcl main argc:$argc"

# EOF #