@echo off
REM #####################################################################
REM File:			install.bat 
REM Author:			M Shaw
REM Purpose:		Portable installer for Ticol Tcl anywhere
REM
REM Versions:
REM 1.00 04/Jan/2022 First version
REM 1.01 29/Jan/2022 Removed /D as this won't overwrite 0 byte files
REM #####################################################################
REM Notes:
REM What it actually copies will depend on the populated host directory
REM #####################################################################
REM Tasks:
REM ADD Independently check target files by date / size?
REM May need to remove xcopy %owdt% as is problematic on some Winvers
REM ADD Check target folder is not same as source folder
REM DONE FIX /D won't overwrite 0 byte (damaged) files. xcopy needs /DS (date+size)
REM		We could test for target file as 0 byte in here but would be complex
REM 	Best alternative would be to wrap into an EXE file or Tcl script
REM DONE ADD check exe dependencies
REM DONE ADD check validity of target folder. Does it exist
REM DONE ADD Check target folder is writeable
REM DONE FIX check DOS ver for %owdt% arg valid
REM DONE FIX /y not valid for NT4
REM >nul 2>&1
REM #####################################################################

ask.exe "  * Ticol Tcl Local Install/Copy Script *  " /f:white /b:magenta /message /crlf
ECHO.

REM Checking prerequisites
IF NOT EXIST ask.exe goto no_ask
IF NOT EXIST filecmp.exe goto no_filecmp
IF NOT EXIST iswrite.exe goto no_iswrite
IF NOT EXIST wver.exe goto no_wver

REM Run...
REM Confirm flag...
set conf=
REM /D overwrite by date (not recommended)
set owdt=
set target=%1

REM Check if target is NULL
if /I "%target%" EQU "" goto syntax
REM Attempt to create the folder, ignoring any errors
MKDIR %target% >nul 2>&1

IF NOT EXIST %target% goto no_target_folder
REM Check if target is writeable...
iswrite.exe %target% 
IF /I %errorlevel% NEQ 0 goto no_write_access

FOR /F "tokens=*" %%i in ('wver.exe /maj') DO SET OSMAJOR=%%i
FOR /F "tokens=*" %%i in ('wver.exe /long') DO SET OSLONG=%%i
ECHO  * Detected Windows version: %oslong%

REM If Win2k+
if /i %OSMAJOR% GEQ 5 (
	REM ECHO Setting YES confirmation
	set conf=/y
)

REM mkdir %target% >nul 2>&1
ECHO  * Installing Ticol Tcl to %target% ...
ECHO.

ECHO Installing Ticol Interpreter Package and DOCs...
xcopy ticol.exe %target% %owdt% %conf% > nul
xcopy ticol.ini %target% %owdt% %conf%  > nul
xcopy autoexec.tcl %target% %owdt% %conf%  > nul
xcopy ticol.man %target% %owdt% %conf%  > nul
xcopy ticol.man.rtf %target% %owdt% %conf%  > nul
xcopy ticol.man.htm %target% %owdt% %conf%  > nul
xcopy ticol.chm %target% %owdt% %conf% > nul
xcopy ticol-quickstart-guide.pdf %target% %owdt% %conf% > nul
xcopy licence.txt %target% %owdt% %conf% > nul
xcopy readme.txt %target% %owdt% %conf% > nul

ECHO Installing Ticol Plugin DLLs...
xcopy ticol_*.dll %target% %owdt% %conf% > nul

ECHO Installing TCL Unit Tests...
xcopy tests.tcl %target% %owdt% %conf% > nul
xcopy test_*.tcl %target% %owdt% %conf% > nul

ECHO Installing TCL Example Files...
xcopy *.tcl %target% %owdt% %conf% > nul

ECHO Installing Windows DLL Test Interface...
xcopy wintest.exe %target% %owdt% %conf% > nul
xcopy richtx32.ocx %target% %owdt% %conf% > nul
xcopy msvbvm50.dll %target% %owdt% %conf% > nul
xcopy ticol.dll %target% %owdt% %conf% > nul
xcopy ticol_stub.exe %target% %owdt% %conf% > nul

ECHO Installing Miscellaneous EXE Files...
xcopy ask.exe %target% %owdt% %conf% > nul
xcopy bf.exe %target% %owdt% %conf% > nul
xcopy crt_test.exe %target% %owdt% %conf% > nul
xcopy diag_expr.exe %target% %owdt% %conf% > nul
xcopy filecmp.exe %target% %owdt% %conf% > nul
xcopy fractal.exe %target% %owdt% %conf% > nul
xcopy hanoi.exe %target% %owdt% %conf% > nul
xcopy iswrite.exe %target% %owdt% %conf% > nul
xcopy md5.exe %target% %owdt% %conf% > nul
xcopy parity.exe %target% %owdt% %conf% > nul
xcopy tcl2c.exe %target% %owdt% %conf% > nul
xcopy tcl2bat.exe %target% %owdt% %conf% > nul
xcopy vbs.exe %target% %owdt% %conf% > nul
xcopy wver.exe %target% %owdt% %conf% > nul
xcopy tdf.exe %target% %owdt% %conf% > nul
xcopy ticol_embed_*.exe %target% %owdt% %conf% > nul

REM xcopy *.exe %target% %owdt% %conf% > nul

ECHO Installing Miscellaneous Files and Unit Test Dependencies....
xcopy *.asm %target% %owdt% %conf% > nul
REM Expand wildcard list
xcopy *.bat %target% %owdt% %conf% > nul
xcopy empty.txt %target% %owdt% %conf% > nul
xcopy lorem.txt %target% %owdt% %conf% > nul
REM Required for Unit tests ... (x2)
xcopy cdecl.dll %target% %owdt% %conf% > nul
xcopy mslib145.dll %target% %owdt% %conf% > nul

xcopy lorem.bmp %target% %owdt% %conf% > nul
xcopy qbf.bmp %target% %owdt% %conf% > nul
xcopy randomstr.bmp %target% %owdt% %conf% > nul
xcopy albums.db %target% %owdt% %conf% > nul
xcopy test.db %target% %owdt% %conf% > nul
xcopy tops.dbf %target% %owdt% %conf% > nul
xcopy xbase_test.dbf %target% %owdt% %conf% > nul

ECHO.
ask.exe "* Done *" /f:white /b:green /message /crlf
goto end

:syntax
REM ask.exe " Ticol Tcl Installer " /f:white /b:magenta /message /crlf 
ECHO Syntax is:  %0 targetdir
ECHO.
ECHO Files will be sourced from the current directory
goto end

:no_ask
ECHO Error: Prerequisite file - ask.exe was not found
goto end

:no_iswrite
ECHO Error: Prerequisite file - iswrite.exe was not found
goto end

:no_wver
ECHO Error: Prerequisite file - wver.exe was not found
goto end

:no_filecmp
ECHO Error: Prerequisite file - filecmp.exe was not found
goto end

:no_target_folder
ECHO Error: Target installation folder doesn't exist
goto end

:no_write_access
ECHO Error: Target installation folder isn't writeable (error %errorlevel%)
goto end

:end
ECHO.

REM EOF ##################################################################